#include "vtkRenderer.h"
#include "vtkQtRenderWindow.h"
#include "vtkQtRenderWindowInteractor.h"
#include "vtkStructuredPoints.h"
#include "vtkStructuredPointsReader.h"
#include "vtkPiecewiseFunction.h"
#include "vtkColorTransferFunction.h"
#include "vtkVolumeProperty.h"
#include "vtkVolumeRayCastCompositeFunction.h"
#include "vtkVolumeRayCastMapper.h"
//#include "vtkVolumeTextureMapper2D.h"
//#include "vtkVolumeProMapper.h"
#include "vtkCamera.h"
#include "vtkVolume.h"

#include <qapplication.h>
#include <qwidget.h>
#include <qpushbutton.h>

#include <iostream>

// Example how to use vtkQtRenderWindow and Interactor with
// your own QApplication and QWidgets

int main( int argc, char *argv[] ) {
    QApplication::setColorSpec(QApplication::CustomColor);
    QApplication a(argc, argv);

    QWidget *qw = new QWidget;
    qw->resize(350, 200);
    qw->show();
    
    // Create the renderer, render window, and interactor
    vtkRenderer *ren1 = vtkRenderer::New();
    // instead of
    // vtkQtRenderWindow *renWin = vtkQtRenderWindow::New();
    // use
    vtkQtRenderWindow *renWin = new vtkQtRenderWindow(qw);
    // use Qt-methods
    renWin->setGeometry(0, 0, 300, 200);
    renWin->show();
    // a pushbutton for this example
    QPushButton *qpb = new QPushButton("Quit", qw);
    qpb->setGeometry(301, 0, 49, 200);
    qpb->show();
    QObject::connect(qpb, SIGNAL(clicked()), qApp, SLOT(quit()));
    
    renWin->AddRenderer(ren1);
    vtkQtRenderWindowInteractor *iren = vtkQtRenderWindowInteractor::New();
    iren->SetRenderWindow(renWin);

    // Read the data from a vtk file
    vtkStructuredPointsReader *reader = vtkStructuredPointsReader::New();
    reader->SetFileName("e:/vtk42/VTKData/Data/ironProt.vtk");
    reader->Update();
    
//    float fa, fb, fc;
    reader->GetOutput()->SetSpacing(0.078, 0.078, 0.78);
//    cout<<fa<<" "<<fb<<" "<<fc<<endl;

    // Create a transfer function mapping scalar value to opacity
    vtkPiecewiseFunction *oTFun = vtkPiecewiseFunction::New();
    oTFun->AddSegment(80, 0.0, 255, 1.0);

    // Create a transfer function mapping scalar value to color (grey)
    vtkColorTransferFunction *cTFun = vtkColorTransferFunction::New();
    cTFun->AddRGBPoint(50, 0.0, 0.0, 0.0);
    cTFun->AddRGBPoint(100, 1.0, 1.0, 1.0);
	

    // Create a property for the volume and set the transfer functions.
    // Turn shading on and use trilinear interpolation
    vtkVolumeProperty *volumeProperty = vtkVolumeProperty::New();
    volumeProperty->SetColor(cTFun);
    volumeProperty->SetScalarOpacity(oTFun);
    volumeProperty->SetInterpolationTypeToLinear();
    volumeProperty->ShadeOn();

    // Create a ray function - this is a compositing ray function
    vtkVolumeRayCastCompositeFunction *compositeFunction = 
        vtkVolumeRayCastCompositeFunction::New();

    // Create the volume mapper and set the ray function and scalar input
    vtkVolumeRayCastMapper *volumeMapper = vtkVolumeRayCastMapper::New();
    volumeMapper->SetVolumeRayCastFunction(compositeFunction);
//    vtkVolumeTextureMapper2D *volumeMapper = vtkVolumeTextureMapper2D::New(); 
    volumeMapper->SetInput(reader->GetOutput());
   
    // Create the volume and set the mapper and property
    vtkVolume *volume = vtkVolume::New();
    volume->SetMapper(volumeMapper);
    volume->SetProperty(volumeProperty);

    // Add this volume to the renderer and get a closer look
    ren1->AddVolume(volume);
    ren1->GetActiveCamera()->Azimuth(20.0);
    ren1->GetActiveCamera()->Dolly(1.60);
    ren1->ResetCameraClippingRange();
 
    // setGeometry was already called, don't use SetSize
    // renWin->SetSize(300,300);
    // Render calls updateGL, this is redundant
    // renWin->Render();

    // Interact with the data at 3 frames per second
    iren->SetDesiredUpdateRate(3.0);
    iren->SetStillUpdateRate(0.001);
    // Start starts a mainloop, we will use our own    
    // iren->Start();
    a.setMainWidget(qw);
    int ret = a.exec();
    
    // Clean up
    ren1->Delete();
    renWin->Delete();
    iren->Delete();
    reader->Delete();
    oTFun->Delete();
    cTFun->Delete();
    volumeProperty->Delete();
//    compositeFunction->Delete();
    volumeMapper->Delete();
    volume->Delete();

    return ret;
}



